import React from 'react'
import {
  FlatList,
  Image,
  StatusBar,
  StyleSheet,
  Text,
  TouchableOpacity,
  View
} from 'react-native'
import { LinearGradient }                  from 'expo'
import { Feather }                         from '@expo/vector-icons'
import { CircleButton, FormationCard, PlayBar, SmallButton } from '../Components'
import { Colors, Images, Metrics }         from '../Themes'
import { connect }                           from 'react-redux'
import { bindActionCreators }                from 'redux'
import { changeFormation }                       from '../Reducers/ProjectActions'
import Modal from 'react-native-modal'

/**
 * Homebase Screen for editing a project
 * -------------------------------------
 * - Add/edit formations and transitions
 * - Label dancers
 * - Preview project
 */
class Overview extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      formations:  [
        [
          [ Metrics.screenWidth * (1/6), Metrics.screenHeight * 0.5 ],
          [ Metrics.screenWidth * (2/6), Metrics.screenHeight * 0.5 ],
          [ Metrics.screenWidth * (3/6), Metrics.screenHeight * 0.5 ],
          [ Metrics.screenWidth * (4/6), Metrics.screenHeight * 0.5 ],
          [ Metrics.screenWidth * (5/6), Metrics.screenHeight * 0.5 ]
        ], {}, {}],
      transitions: [{}, {}],
      isModalVisible: false,
    }
  }

  _toggleModal = () => {
    this.setState({ isModalVisible: !this.state.isModalVisible });
  }

  /* Navigation options */
  static navigationOptions = ({ navigation }) => {
    return {
      title: 'OVERVIEW',
      headerTitleStyle: {
        textTransform: 'uppercase',
        color: Colors.snow,
        fontWeight: 'normal',
      },
      headerStyle: {
        backgroundColor: Colors.background,
        borderBottomWidth: 0,
        marginVertical: Metrics.baseMargin,
        marginHorizontal: Metrics.doubleBaseMargin,
      },
      headerLeft:
        <SmallButton
          text='HOME'
          onPress={() => navigation.navigate('Home')}
        />
      ,
      headerRight:
        <View style={{flexDirection: 'row'}}>
          <SmallButton
            text='LABEL'
            style={{marginHorizontal: Metrics.baseMargin}}
          />
          <SmallButton
            text='PREVIEW'
          />
        </View>
    };
  }

  /* Set orientation to landscape */
  componentDidMount() {
    Expo.ScreenOrientation.allowAsync(Expo.ScreenOrientation.Orientation.LANDSCAPE);
    StatusBar.setHidden(true);
  }

  /* Revert orientation to portrait */
  componentWillUnmount() {
    Expo.ScreenOrientation.allowAsync(Expo.ScreenOrientation.Orientation.PORTRAIT);
  }

  separator = () => {
    return(
      <View style={{width: Metrics.doubleBaseMargin, backgroundColor: Colors.background}}/>
    );
  }

  /* Add padding between formations and a transition */
  formationSeparator = () => {
    return (
        <View
          style={{
            justifyContent: 'center',
            alignItems: 'center',
            backgroundColor:Colors.background,
            margin: Metrics.baseMargin
          }}
        >
          <TouchableOpacity style={{justifyContent: 'center', alignItems: 'center'}} onPress={() => this.props.navigation.navigate('Transition')}>
            <LinearGradient
              colors={[Colors.darkGradient, Colors.midGradient, Colors.lightGradient]}
              style={styles.circle}
              start={[0, 1]}
              end={[1, 0]}
              locations={[0, 0.5, 1.3]}
            >
              <Feather name='plus' size={Metrics.icons.small} color={Colors.lightGray}/>
            </LinearGradient>
          </TouchableOpacity>
        </View>
    );
  }

  /* Empty formation that links to Create screen */
  renderFormationCard = ({index}) => {
    var formation;
    switch(index) {
      case 0:
        formation = 0;
        break;
      case 1:
        formation = 1;
        break;
      case 2:
        if (!this.props.projects.taskOneDone) {
          formation = 2;
        }
        else {
          formation = 3;
        }
        break;
    }

    return (
      <View style={{flexDirection: 'row'}}>
        {(index === 0) && this.separator()}
        <FormationCard
          index={formation}
          onPress={() => this.props.navigation.navigate('Create', {
            formations: this.state.formations[index],
            formationNum: index + 1,
          })}
        />
        {(index === this.state.formations.length - 1) && this.separator()}
      </View>
    )
  }

  savedThirdFormation = () => {
    this.setState({savedThirdFormation:true});
  }

  modal = () => {
    msg = 'Switching formations will remove the\ntransition you created. Do you want to\nproceed?';
    return (
    <Modal isVisible={this.state.isModalVisible}>
      <View style={{ flex: 1}}>
        <TouchableOpacity style={{flex:1, justifyContent: 'center', alignItems: 'center'}} onPress={this._toggleModal}>
          <View style={styles.modal}>
            <Text style={styles.text}>{msg}</Text>
            <SmallButton
              text='CANCEL'
              onPress={this._toggleModal}
              style={{position: 'absolute', bottom: Metrics.baseMargin, left: Metrics.doubleBaseMargin}}
            />
            <SmallButton
              text='CONTINUE'
              style={{position: 'absolute', bottom: Metrics.baseMargin, right: Metrics.doubleBaseMargin}}
            />
          </View>
        </TouchableOpacity>
      </View>
    </Modal>
  );
  }

  render() {
    return (
      <View style={styles.container}>
        {this.state.isModalVisible && this.modal()}
        <View style={styles.listContainer}>
          <FlatList
            data={this.state.formations}
            renderItem={this.renderFormationCard}
            keyExtractor={(item, index) => index.toString()}
            horizontal={true}
            ItemSeparatorComponent={this.formationSeparator}
            extraData={this.props.projects}
          />
        </View>
        <PlayBar _toggleModal={this._toggleModal}/>
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    justifyContent: 'flex-end',
    backgroundColor: Colors.background,
  },
  listContainer: {
    flex: 1,
    flexDirection: 'row',
    paddingVertical: Metrics.doubleBaseMargin,
  },
  circle: {
    borderRadius: (Metrics.icons.medium + Metrics.baseMargin*2) / 2,
    padding: Metrics.baseMargin,
    backgroundColor: Colors.darkGradient,
  },
  modal: {
    width: 393,
    height: 237,
    backgroundColor: Colors.background,
    borderRadius: 12,
    justifyContent: 'center',
    alignItems: 'center',
  },
  text: {
    textAlign: 'center',
    fontSize: Metrics.text.header,
    color: Colors.snow,
  },
});

const mapStateToProps = (state) => {
  const { projects } = state
  return { projects }
};

const mapDispatchToProps = dispatch => (
  bindActionCreators({
    changeFormation,
  }, dispatch)
);

export default connect(mapStateToProps, mapDispatchToProps)(Overview);
