//
//  Formation.swift
//  ARScreen
//
//  Created by Sarina Wu on 12/3/18.
//  Copyright © 2018 LetsDance. All rights reserved.
//

import Foundation
import SceneKit

let transparentHighlightColor = UIColor(red:0.89, green:0.35, blue:0.99, alpha:0.6)
let highlightColor = UIColor(red:0.89, green:0.35, blue:0.99, alpha:1.0)
let secondaryColor = UIColor(red:0.35, green:0.00, blue:0.47, alpha:1.0)

//represents one dancer
//can display the representation for one dancer
class Dancer {
    var name: String
    var x: Float
    var z: Float
    var node: SCNNode
    
    init(_ name: String, x: Float, z: Float) {
        self.name = name
        self.x = x
        self.z = z
        self.node = SCNNode()
        self.getNode()
    }
    
    func getNode() {
        let head = SCNSphere(radius: 0.2)
        head.materials.first?.diffuse.contents = transparentHighlightColor
        let headNode = SCNNode(geometry: head)
        headNode.position = SCNVector3(x: x, y: 1.5, z: z)
        
        let body = SCNCone(topRadius: 0.2, bottomRadius: 0, height: 1.2)
        body.materials.first?.diffuse.contents = secondaryColor
        let bodyNode = SCNNode(geometry: body)
        bodyNode.position = SCNVector3(x: x, y: 0.60, z: z)
        
        let base = SCNCylinder(radius: 0.2, height: 0.05)
        base.materials.first?.diffuse.contents = highlightColor
        let baseNode = SCNNode(geometry: base)
        baseNode.position = SCNVector3(x: x, y: 0, z: z)
        
        let textNode = createTextNode(string: name)
        textNode.position = SCNVector3(x: x, y: 1.35, z: z)
        
        let dancerNode = SCNNode()
        dancerNode.name = name
        dancerNode.addChildNode(headNode)
        dancerNode.addChildNode(bodyNode)
        dancerNode.addChildNode(baseNode)
        dancerNode.addChildNode(textNode)
        self.node = dancerNode
    }
    
    func createTextNode(string: String) -> SCNNode {
        let text = SCNText(string: string, extrusionDepth: 0.1)
        text.font = UIFont.systemFont(ofSize: 10.0)
        text.flatness = 0.01
        text.firstMaterial?.diffuse.contents = UIColor.white
        
        let textNode = SCNNode(geometry: text)
        
        let fontSize = Float(0.04)
        textNode.scale = SCNVector3(fontSize, fontSize, fontSize)
        
        //set pivot of text
        var minVec = SCNVector3Zero
        var maxVec = SCNVector3Zero
        (minVec, maxVec) =  textNode.boundingBox
        textNode.pivot = SCNMatrix4MakeTranslation(
            minVec.x + (maxVec.x - minVec.x)/2,
            minVec.y,
            minVec.z + (maxVec.z - minVec.z)/2
        )
        
        return textNode
    }
    
    func reset() {
        node.position = SCNVector3(x: x, y: 0.0, z: z)
    }
}

class Formation {
    var dancers: [Dancer]
    var name: String
    
    init(named name: String, withDancers dancers: [Dancer]) {
        self.name = name
        self.dancers = dancers
    }
    
    init(_ name: String, withNameDictionary formDict: [String: [Float]]) {
        self.name = name
        self.dancers = []
        for name in formDict.keys {
            let position = formDict[name]!
            let dancer = Dancer(name, x: position[0], z: position[1])
            self.dancers.append(dancer)
        }
    }
    
    //generates a node that displays the formation at a given position
    func generateFormation(atPosition position: SCNVector3) -> SCNNode {
        let formationNode = SCNNode()
        formationNode.name = name
        formationNode.position = position
        for dancer in dancers {
            formationNode.addChildNode(dancer.node)
        }
        return formationNode
    }
}

//groups formations into a choreography
//automatically calculates transitions
//assumes identical indexing for dancers across formations
class Choreo {
    var formations: [Formation]
    var transitions: [Double]
    var name: String
    var choreoNode: SCNNode
    var animations: [SCNAction]
    let animationKey: String = "choreoAnimation"
    
    //transitions are indexed in order of formations, where the
    //value at index 0 is the transition time between Formation 0 and formation 1
    init(_ name: String, withFormations formations: [Formation], transitionLengths transitions: [Double]) {
        self.formations = formations
        self.transitions = transitions
        self.name = name
        self.choreoNode = SCNNode()
        self.animations = []
        self.generateChoreo()
    }
    
    // loads each dancer and each dancer's animations
    func generateChoreo(){
        if formations.count == 0 {
            debugPrint("no formations")
            return
        }
        self.choreoNode.name = name
        
        let wait = SCNAction.wait(duration: 3)
        let startDancers = formations[0].dancers
        //for every dancer
        for d in 0..<startDancers.count {
            let dancer = startDancers[d]
            let dancerNode = dancer.node
            
            var dancerAnimationList: [SCNAction] = []
            
            //calculate the animation to the next formation
            for i in 1..<formations.count {
                let prevFormation = formations[i-1]
                let nextFormation = formations[i]
                
                let prevX = prevFormation.dancers[d].x
                let prevZ = prevFormation.dancers[d].z
                
                let nextX = nextFormation.dancers[d].x
                let nextZ = nextFormation.dancers[d].z
                
                let movement = SCNVector3(x: nextX-prevX, y: 0.0, z: nextZ-prevZ)
                let action = SCNAction.move(by: movement, duration: transitions[i-1])
                
                dancerAnimationList.append(wait)
                dancerAnimationList.append(action)
            }
            dancerAnimationList.append(wait)

            let dancerAnimationSequence = SCNAction.sequence(dancerAnimationList)
            let reverseSequence = SCNAction.reversed(dancerAnimationSequence)
            let loopAction = SCNAction.sequence([dancerAnimationSequence, reverseSequence()])
            let repeatForever = SCNAction.repeatForever(loopAction)
            self.animations.append(repeatForever)
            
            self.choreoNode.addChildNode(dancerNode)
        }
    }
    
    // play animations
    func play() {
        let startDancers = formations[0].dancers
        for d in 0..<startDancers.count {
            let dancerNode = startDancers[d].node
            let animation = self.animations[d]
            dancerNode.runAction(animation, forKey: self.animationKey)
        }
    }
    
    // stop animations
    func pause() {
        let startDancers = formations[0].dancers
        for d in 0..<startDancers.count {
            let dancer = startDancers[d]
            let dancerNode = dancer.node
            dancerNode.removeAction(forKey: self.animationKey)
            dancer.reset()
        }
    }
}
