//
//  ViewController.swift
//  LetsDanceAR
//
//  Created by Sarina Wu on 12/5/18.
//  Copyright © 2018 LetsDance. All rights reserved.
//

import UIKit
import SceneKit
import ARKit

extension simd_float4x4 {
    func position() -> SCNVector3 {
        return SCNVector3(columns.3.x, columns.3.y, columns.3.z)
    }
}

class ViewController: UIViewController, ARSCNViewDelegate {
    @IBOutlet weak var messageView: UIView!
    @IBOutlet weak var scanMessageLabel: UILabel!
    @IBOutlet weak var tapMessageLabel: UILabel!
    @IBOutlet weak var playButton: UIBarButtonItem!
    @IBOutlet weak var toolbar: UIToolbar!
    @IBOutlet weak var navigationBar: UINavigationBar!
    
    
    var playBtn = UIBarButtonItem()
    var pauseBtn = UIBarButtonItem()
    let highlightColor = UIColor(red:0.89, green:0.35, blue:0.99, alpha:1.0)
    let font = UIFont(name: "Montserrat-Medium", size: 18)
    
    let song = Bundle.main.path(forResource: "art.scnassets/fractures.mp3", ofType:nil)!
    var audioPlayer = AVAudioPlayer()
    var audioPlayable = true
    
    @IBOutlet var sceneView: ARSCNView!
    let formDicts: [String: [String: [Float]]] = [
        "form1": ["A": [-2,0],
                  "B": [-1,0],
                  "C": [0,0],
                  "D": [1,0],
                  "E": [2,0]],
        
        "form2": ["A": [-1.5,0],
                  "B": [-0.75,0.5],
                  "C": [0,0],
                  "D": [0.75,0.5],
                  "E": [1.5,0]],
        
        "form3": ["A": [-1.5,0.5],
                  "B": [-0.75,0],
                  "C": [0,0.5],
                  "D": [0.75,0],
                  "E": [1.5,0.5]]]
    var formations: [Formation] = []
    let transitions: [Double] = [1.0, 1.0]
    let choreoName = "choreo"
    var foundPlanes = false
    var tapped = false
    var choreo: Choreo?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Set the view's delegate
        sceneView.delegate = self
        
        // Show statistics such as fps and timing information
//        sceneView.showsStatistics = true
        
        // Create a new scene
        let scene = SCNScene()
        
        // Set the scene to the view
        sceneView.scene = scene
        sceneView.delegate = self
        //sceneView.debugOptions = [ARSCNDebugOptions.showFeaturePoints]
        
        setUpMessageView()
        addTapGestureToSceneView()
        configureLighting()
        initializeChoreo()
        setUpToolbar()
        setUpAudio()
    }
    
    func setUpMessageView() {
        messageView.layer.cornerRadius = 10.0
        messageView.layer.shadowColor = UIColor.black.cgColor
        messageView.layer.shadowOpacity = 0.8
        messageView.layer.shadowRadius = 4.0
        messageView.layer.shadowOffset = CGSize(width: 4.0, height: 4.0)
        messageView.clipsToBounds = true
        messageView.layer.masksToBounds = false
        sceneView.addSubview(messageView)
        messageView.isHidden = false
        
        scanMessageLabel.font = font
        tapMessageLabel.font = font
    }
    
    func initializeChoreo() {
        for name in formDicts.keys {
            let formation = Formation(name, withNameDictionary: formDicts[name]!)
            formations.append(formation)
        }
        
    }

    func configureLighting() {
        sceneView.autoenablesDefaultLighting = true
        sceneView.automaticallyUpdatesLighting = true
    }
    
    func setUpToolbar() {
        self.toolbar.setBackgroundImage(UIImage(),
                                        forToolbarPosition: .any,
                                        barMetrics: .default)
        self.toolbar.setShadowImage(UIImage(), forToolbarPosition: .any)
        
        navigationBar.setBackgroundImage(UIImage(), for: .default)
        navigationBar.shadowImage = UIImage()
        navigationBar.isTranslucent = true
        self.navigationController?.view.backgroundColor = .clear
        
        self.playBtn = UIBarButtonItem(barButtonSystemItem: .play , target: self, action: #selector(play(sender:)))
        self.playBtn.tintColor = highlightColor
        
        self.pauseBtn = UIBarButtonItem(barButtonSystemItem: .pause , target: self, action: #selector(pause(sender:)))
        self.pauseBtn.tintColor = highlightColor
        
        var toolbarItems = self.toolbar.items
        toolbarItems?[0] = playBtn
        self.toolbar.setItems(toolbarItems, animated: false)
    }
    
    @objc func setUpAudio() {
        do {
            audioPlayer = try AVAudioPlayer(contentsOf: URL(fileURLWithPath: song))
        }
        catch{
            audioPlayable = false
            print(error)
        }
    }
    
    @objc func addFormationToSceneView(withGestureRecognizer recognizer: UIGestureRecognizer) {
        debugPrint("tap started")
        
        let tapLocation = recognizer.location(in: sceneView)
        let hitTestResults = sceneView.hitTest(tapLocation, types: .existingPlaneUsingExtent)
        
        guard let hitTestResult = hitTestResults.first else { return }
        
        if(!tapped && foundPlanes){
            tapped = true
            messageView.removeFromSuperview()
        }
        
        let translation = hitTestResult.worldTransform.position()
        let x = translation.x
        let y = translation.y
        let z = translation.z
        
        self.choreo = Choreo(choreoName, withFormations: formations, transitionLengths: transitions)
        
        let choreoNode = self.choreo!.choreoNode
        choreoNode.position = SCNVector3(x: x, y: y, z: z)
        
        if let oldNode = sceneView.scene.rootNode.childNode(withName: choreoName, recursively: true) {
            sceneView.scene.rootNode.replaceChildNode(oldNode, with: choreoNode)
        } else {
            sceneView.scene.rootNode.addChildNode(choreoNode)
        }
        
        debugPrint("tap recognized")
    }
    
    @objc func play(sender: UIBarButtonItem) {
        debugPrint("play pressed")
        if foundPlanes && tapped && self.choreo != nil {
            self.choreo?.play()
            var toolbarItems = self.toolbar.items
            toolbarItems?[0] = pauseBtn
            self.toolbar.setItems(toolbarItems, animated: false)
            
            if audioPlayable {
                audioPlayer.play()
                debugPrint("playing audio")
            }
        }
    }
    
    @objc func pause(sender: UIBarButtonItem) {
        debugPrint("pause pressed")
        if foundPlanes && tapped && self.choreo != nil {
            self.choreo?.pause()
            var toolbarItems = self.toolbar.items
            toolbarItems?[0] = playBtn
            self.toolbar.setItems(toolbarItems, animated: false)
            
            if audioPlayable {
                audioPlayer.stop()
                audioPlayer.prepareToPlay()
            }
        }
    }
    
    func addTapGestureToSceneView() {
        let tapGestureRecognizer = UITapGestureRecognizer(target: self, action: #selector(ViewController.addFormationToSceneView(withGestureRecognizer:)))
        sceneView.addGestureRecognizer(tapGestureRecognizer)
    }
    
//    let feedbackGenerator = UIImpactFeedbackGenerator(style: .medium)
    
    func renderer(_ renderer: SCNSceneRenderer, didAdd node: SCNNode, for anchor: ARAnchor) {
        guard let planeAnchor = anchor as? ARPlaneAnchor else { return }
        
//        feedbackGenerator.impactOccurred()
        if(!foundPlanes && !tapped) {
            foundPlanes = true
            scanMessageLabel.isHidden = true
            tapMessageLabel.isHidden = false
        }
        
        let width = CGFloat(planeAnchor.extent.x)
        let height = CGFloat(planeAnchor.extent.z)
        let plane = SCNPlane(width: width, height: height)
        
        let grid = UIImage(named: "art.scnassets/plane_grid.png")
        let material = SCNMaterial()
        material.diffuse.contents = grid
        plane.materials = [material]
        plane.firstMaterial?.transparency = 0.5
        
        let planeNode = SCNNode(geometry: plane)
        
        let x = CGFloat(planeAnchor.center.x)
        let y = CGFloat(planeAnchor.center.y)
        let z = CGFloat(planeAnchor.center.z)
        planeNode.position = SCNVector3(x,y,z)
        planeNode.eulerAngles.x = -.pi / 2
        
        node.addChildNode(planeNode)
    }
    
    func renderer(_ renderer: SCNSceneRenderer, didUpdate node: SCNNode, for anchor: ARAnchor) {
        guard let planeAnchor = anchor as?  ARPlaneAnchor,
            let planeNode = node.childNodes.first,
            let plane = planeNode.geometry as? SCNPlane
            else { return }
        
        let width = CGFloat(planeAnchor.extent.x)
        let height = CGFloat(planeAnchor.extent.z)
        plane.width = width
        plane.height = height

        let x = CGFloat(planeAnchor.center.x)
        let y = CGFloat(planeAnchor.center.y)
        let z = CGFloat(planeAnchor.center.z)
        planeNode.position = SCNVector3(x, y, z)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        // Create a session configuration
        let configuration = ARWorldTrackingConfiguration()

        configuration.planeDetection = .horizontal
        
        // Run the view's session
        sceneView.session.run(configuration)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        pause(sender: UIBarButtonItem())
        // Pause the view's session
        sceneView.session.pause()
    }

    // MARK: - ARSCNViewDelegate
    
/*
    // Override to create and configure nodes for anchors added to the view's session.
    func renderer(_ renderer: SCNSceneRenderer, nodeFor anchor: ARAnchor) -> SCNNode? {
        let node = SCNNode()
     
        return node
    }
*/
    
    func session(_ session: ARSession, didFailWithError error: Error) {
        // Present an error message to the user
        
    }
    
    func sessionWasInterrupted(_ session: ARSession) {
        // Inform the user that the session has been interrupted, for example, by presenting an overlay
        
    }
    
    func sessionInterruptionEnded(_ session: ARSession) {
        // Reset tracking and/or remove existing anchors if consistent tracking is required
        
    }
}
